#include <cstring>
#include "gp_io.h"
#include "gp_gfx.h"

#define INPUT_COUNT 64
#define INPUT_MAX 1024
#define OUTPUT_COUNT 64
#define OUTPUT_MAX 256

static char input_buffer[INPUT_COUNT][INPUT_MAX];
int input_start;
int input_end;

static char output_buffer[OUTPUT_COUNT][OUTPUT_MAX];
int output_start;
int output_end;

void gp_initIO()
{
	input_start = input_end = 0;
	output_start = output_end = 0;
}

int gp_available()
{
	return input_start!=input_end;
}

void increment(int* v,int max)
{
	if (*v+1 == max)
		*v = 0;
	else
		*v += 1;
}

char *gp_input()
{
	if (!gp_available())
		return NULL;
	char* ret = input_buffer[input_start];
	increment(&input_start,INPUT_COUNT);
	return ret;
}

void gp_write(const char* line)
{
	if (line==NULL)
		error("gp_write: NULL");
	strncpy(input_buffer[input_end],line,INPUT_MAX-1);
	increment(&input_end,INPUT_COUNT);
	if (input_start==input_end)
		error("LOOP gp_write");
}

void gp_output(const char* line)
{
	if (line==NULL)
		error("gp_output: NULL");
	if (strncmp(line,"info",4)==0) // skip infos
		return;
	strncpy(output_buffer[output_end],line,OUTPUT_MAX-1);
	increment(&output_end,OUTPUT_COUNT);
	if (output_start==output_end)
		error("LOOP gp_output");
}

char *gp_read()
{
	if (output_start==output_end)
		return NULL;
	char* ret = output_buffer[output_start];
	increment(&output_start,OUTPUT_COUNT);
	return ret;
}
